@@ -56,7 +56,7 @@
 #define TAG CLIENT_TAG("android")
 
 /* Defines the JNI version supported by this library. */
-#define FREERDP_JNI_VERSION "3.4.0"
+#define FREERDP_JNI_VERSION FREERDP_VERSION_FULL
 static void android_OnChannelConnectedEventHandler(void* context,
                                                    const ChannelConnectedEventArgs* e)
 {
@@ -182,23 +182,20 @@ static BOOL android_desktop_resize(rdpContext* context)
 
 static BOOL android_pre_connect(freerdp* instance)
 {
-	int rc;
-	rdpSettings* settings;
-
 	WINPR_ASSERT(instance);
 	WINPR_ASSERT(instance->context);
 
-	settings = instance->context->settings;
+	rdpSettings* settings = instance->context->settings;
 
 	if (!settings)
 		return FALSE;
 
-	rc = PubSub_SubscribeChannelConnected(instance->context->pubSub,
-	                                      android_OnChannelConnectedEventHandler);
+	int rc = PubSub_SubscribeChannelConnected(instance->context->pubSub,
+	                                          android_OnChannelConnectedEventHandler);
 
 	if (rc != CHANNEL_RC_OK)
 	{
-		WLog_ERR(TAG, "Could not subscribe to connect event handler [%l08X]", rc);
+		WLog_ERR(TAG, "Could not subscribe to connect event handler [%08X]", rc);
 		return FALSE;
 	}
 
@@ -207,7 +204,7 @@ static BOOL android_pre_connect(freerdp* instance)
 
 	if (rc != CHANNEL_RC_OK)
 	{
-		WLog_ERR(TAG, "Could not subscribe to disconnect event handler [%l08X]", rc);
+		WLog_ERR(TAG, "Could not subscribe to disconnect event handler [%08X]", rc);
 		return FALSE;
 	}
 
@@ -448,8 +445,8 @@ static int android_freerdp_run(freerdp* instance)
 
 		if (status == WAIT_FAILED)
 		{
-			WLog_ERR(TAG, "WaitForMultipleObjects failed with %" PRIu32 " [%08lX]", status,
-			         GetLastError());
+			WLog_ERR(TAG, "WaitForMultipleObjects failed with %u [%08X]", status,
+			         (unsigned)GetLastError());
 			break;
 		}
 
@@ -500,7 +497,7 @@ static DWORD WINAPI android_thread_func(LPVOID param)
 	else
 	{
 		status = android_freerdp_run(instance);
-		WLog_DBG(TAG, "Disonnect...");
+		WLog_DBG(TAG, "Disconnect...");
 
 		if (!freerdp_disconnect(instance))
 			status = GetLastError();
@@ -650,7 +647,7 @@ JNIEXPORT jlong JNICALL Java_com_freerdp_freerdpcore_services_LibFreeRDP_freerdp
 	if (setenv("HOME", _strdup(envStr), 1) != 0)
 	{
 		char ebuffer[256] = { 0 };
-		WLog_FATAL(TAG, "Failed to set environemnt HOME=%s %s [%d]", env,
+		WLog_FATAL(TAG, "Failed to set environment HOME=%s %s [%d]", envStr,
 		           winpr_strerror(errno, ebuffer, sizeof(ebuffer)), errno);
 		return (jlong)NULL;
 	}
@@ -732,15 +729,14 @@ JNIEXPORT jboolean JNICALL Java_com_freerdp_freerdpcore_services_LibFreeRDP_free
     JNIEnv* env, jclass cls, jlong instance)
 {
 	freerdp* inst = (freerdp*)instance;
-	androidContext* ctx;
 
 	if (!inst || !inst->context)
 	{
-		WLog_FATAL(TAG, "(env=%p, cls=%p, instance=%d", (void*)env, (void*)cls, instance);
+		WLog_FATAL(TAG, "(env=%p, cls=%p, instance=%lld", (void*)env, (void*)cls, instance);
 		return JNI_FALSE;
 	}
 
-	ctx = (androidContext*)inst->context;
+	androidContext* ctx = (androidContext*)inst->context;
 
 	if (!(ctx->thread = CreateThread(NULL, 0, android_thread_func, inst, 0, NULL)))
 	{
@@ -754,17 +750,15 @@ JNIEXPORT jboolean JNICALL Java_com_freerdp_freerdpcore_services_LibFreeRDP_free
     JNIEnv* env, jclass cls, jlong instance)
 {
 	freerdp* inst = (freerdp*)instance;
-	androidContext* ctx;
-	ANDROID_EVENT* event;
 
 	if (!inst || !inst->context || !cls || !env)
 	{
-		WLog_FATAL(TAG, "(env=%p, cls=%p, instance=%d", (void*)env, (void*)cls, instance);
+		WLog_FATAL(TAG, "(env=%p, cls=%p, instance=%lld", (void*)env, (void*)cls, instance);
 		return JNI_FALSE;
 	}
 
-	ctx = (androidContext*)inst->context;
-	event = (ANDROID_EVENT*)android_event_disconnect_new();
+	androidContext* ctx = (androidContext*)inst->context;
+	ANDROID_EVENT* event = (ANDROID_EVENT*)android_event_disconnect_new();
 
 	if (!event)
 		return JNI_FALSE;
@@ -798,7 +792,7 @@ Java_com_freerdp_freerdpcore_services_LibFreeRDP_freerdp_1update_1graphics(JNIEn
 
 	if (!env || !cls || !inst)
 	{
-		WLog_FATAL(TAG, "(env=%p, cls=%p, instance=%d", (void*)env, (void*)cls, instance);
+		WLog_FATAL(TAG, "(env=%p, cls=%p, instance=%lld", (void*)env, (void*)cls, instance);
 		return JNI_FALSE;
 	}
 
@@ -996,7 +990,7 @@ jint JNI_OnLoad(JavaVM* vm, void* reserved)
 {
 	JNIEnv* env;
 	setlocale(LC_ALL, "");
-	WLog_DBG(TAG, "Setting up JNI environement...");
+	WLog_DBG(TAG, "Setting up JNI environment...");
 
 	/*
 	    if (freerdp_handle_signals() != 0)
@@ -1029,7 +1023,7 @@ jint JNI_OnLoad(JavaVM* vm, void* reserved)
 void JNICALL JNI_OnUnload(JavaVM* vm, void* reserved)
 {
 	JNIEnv* env;
-	WLog_DBG(TAG, "Tearing down JNI environement...");
+	WLog_DBG(TAG, "Tearing down JNI environment...");
 
 	if ((*vm)->GetEnv(vm, (void**)&env, JNI_VERSION_1_6) != JNI_OK)
 	{
