@@ -162,7 +162,8 @@ static int freerdp_peer_virtual_channel_write(freerdp_peer* client, HANDLE hChan
 
 	while (length > 0)
 	{
-		s = rdp_send_stream_init(rdp);
+		UINT16 sec_flags = 0;
+		s = rdp_send_stream_init(rdp, &sec_flags);
 
 		if (!s)
 			return -1;
@@ -192,7 +193,7 @@ static int freerdp_peer_virtual_channel_write(freerdp_peer* client, HANDLE hChan
 		Stream_Write(s, buffer, chunkSize);
 
 		WINPR_ASSERT(peerChannel->channelId <= UINT16_MAX);
-		if (!rdp_send(rdp, s, (UINT16)peerChannel->channelId))
+		if (!rdp_send(rdp, s, (UINT16)peerChannel->channelId, sec_flags))
 			return -1;
 
 		buffer += chunkSize;
@@ -274,6 +275,8 @@ static BOOL freerdp_peer_initialize(freerdp_peer* client)
 	}
 
 	nego_set_RCG_supported(rdp->nego, settings->RemoteCredentialGuard);
+	nego_set_restricted_admin_mode_supported(rdp->nego, settings->RestrictedAdminModeSupported);
+
 	if (!rdp_server_transition_to_state(rdp, CONNECTION_STATE_INITIAL))
 		return FALSE;
 
@@ -430,23 +433,21 @@ static state_run_t peer_recv_data_pdu(freerdp_peer* client, wStream* s,
 static state_run_t peer_recv_tpkt_pdu(freerdp_peer* client, wStream* s)
 {
 	state_run_t rc = STATE_RUN_SUCCESS;
-	rdpRdp* rdp = NULL;
 	UINT16 length = 0;
 	UINT16 pduType = 0;
 	UINT16 pduSource = 0;
 	UINT16 channelId = 0;
 	UINT16 securityFlags = 0;
-	rdpSettings* settings = NULL;
 
 	WINPR_ASSERT(s);
 	WINPR_ASSERT(client);
 	WINPR_ASSERT(client->context);
 
-	rdp = client->context->rdp;
+	rdpRdp* rdp = client->context->rdp;
 	WINPR_ASSERT(rdp);
 	WINPR_ASSERT(rdp->mcs);
 
-	settings = client->context->settings;
+	rdpSettings* settings = client->context->settings;
 	WINPR_ASSERT(settings);
 
 	if (!rdp_read_header(rdp, s, &length, &channelId))
@@ -492,7 +493,7 @@ static state_run_t peer_recv_tpkt_pdu(freerdp_peer* client, wStream* s)
 			case PDU_TYPE_FLOW_TEST:
 				if (!Stream_SafeSeek(s, remain))
 				{
-					WLog_WARN(TAG, "Short PDU, need %" PRIuz " bytes, got %" PRIuz, remain,
+					WLog_WARN(TAG, "Short PDU, need %" PRIu16 " bytes, got %" PRIuz, remain,
 					          Stream_GetRemainingLength(s));
 					return STATE_RUN_FAILED;
 				}
@@ -758,7 +759,7 @@ static state_run_t rdp_peer_handle_state_active(freerdp_peer* client)
 	}
 	if (!client->connected)
 	{
-		WLog_ERR(TAG, "PostConnect for peer %p failed", client);
+		WLog_ERR(TAG, "PostConnect for peer %p failed", WINPR_CXX_COMPAT_CAST(const void*, client));
 		ret = STATE_RUN_FAILED;
 	}
 	else if (!client->activated)
@@ -773,7 +774,8 @@ static state_run_t rdp_peer_handle_state_active(freerdp_peer* client)
 
 		if (!activated)
 		{
-			WLog_ERR(TAG, "Activate for peer %p failed", client);
+			WLog_ERR(TAG, "Activate for peer %p failed",
+			         WINPR_CXX_COMPAT_CAST(const void*, client));
 			ret = STATE_RUN_FAILED;
 		}
 		else
@@ -787,27 +789,26 @@ static state_run_t rdp_peer_handle_state_active(freerdp_peer* client)
 static state_run_t peer_recv_callback_internal(WINPR_ATTR_UNUSED rdpTransport* transport,
                                                wStream* s, void* extra)
 {
-	UINT32 SelectedProtocol = 0;
 	freerdp_peer* client = (freerdp_peer*)extra;
-	rdpRdp* rdp = NULL;
 	state_run_t ret = STATE_RUN_FAILED;
-	rdpSettings* settings = NULL;
 
 	WINPR_ASSERT(transport);
 	WINPR_ASSERT(client);
 	WINPR_ASSERT(client->context);
 
-	rdp = client->context->rdp;
+	rdpRdp* rdp = client->context->rdp;
 	WINPR_ASSERT(rdp);
 
-	settings = client->context->settings;
+	rdpSettings* settings = client->context->settings;
 	WINPR_ASSERT(settings);
 
 	IFCALL(client->ReachedState, client, rdp_get_state(rdp));
 	switch (rdp_get_state(rdp))
 	{
 		case CONNECTION_STATE_INITIAL:
-			if (rdp_server_transition_to_state(rdp, CONNECTION_STATE_NEGO))
+			if (!freerdp_settings_enforce_consistency(settings))
+				ret = STATE_RUN_FAILED;
+			else if (rdp_server_transition_to_state(rdp, CONNECTION_STATE_NEGO))
 				ret = STATE_RUN_CONTINUE;
 			break;
 
@@ -818,28 +819,37 @@ static state_run_t peer_recv_callback_internal(WINPR_ATTR_UNUSED rdpTransport* t
 			}
 			else
 			{
-				SelectedProtocol = nego_get_selected_protocol(rdp->nego);
+				const UINT32 SelectedProtocol = nego_get_selected_protocol(rdp->nego);
+
 				settings->RdstlsSecurity = (SelectedProtocol & PROTOCOL_RDSTLS) ? TRUE : FALSE;
 				settings->NlaSecurity = (SelectedProtocol & PROTOCOL_HYBRID) ? TRUE : FALSE;
 				settings->TlsSecurity = (SelectedProtocol & PROTOCOL_SSL) ? TRUE : FALSE;
 				settings->RdpSecurity = (SelectedProtocol == PROTOCOL_RDP) ? TRUE : FALSE;
 
+				client->authenticated = FALSE;
 				if (SelectedProtocol & PROTOCOL_HYBRID)
 				{
 					SEC_WINNT_AUTH_IDENTITY_INFO* identity =
 					    (SEC_WINNT_AUTH_IDENTITY_INFO*)nego_get_identity(rdp->nego);
-					sspi_CopyAuthIdentity(&client->identity, identity);
-					IFCALLRET(client->Logon, client->authenticated, client, &client->identity,
-					          TRUE);
+					if (sspi_CopyAuthIdentity(&client->identity, identity) >= 0)
+					{
+						client->authenticated =
+						    IFCALLRESULT(TRUE, client->Logon, client, &client->identity, TRUE);
+					}
 					nego_free_nla(rdp->nego);
 				}
 				else
 				{
-					IFCALLRET(client->Logon, client->authenticated, client, &client->identity,
-					          FALSE);
+					client->authenticated =
+					    IFCALLRESULT(TRUE, client->Logon, client, &client->identity, FALSE);
+				}
+				if (!client->authenticated)
+					ret = STATE_RUN_FAILED;
+				else
+				{
+					if (rdp_server_transition_to_state(rdp, CONNECTION_STATE_MCS_CREATE_REQUEST))
+						ret = STATE_RUN_SUCCESS;
 				}
-				if (rdp_server_transition_to_state(rdp, CONNECTION_STATE_MCS_CREATE_REQUEST))
-					ret = STATE_RUN_SUCCESS;
 			}
 			break;
 
@@ -1145,7 +1155,8 @@ static state_run_t peer_recv_callback_internal(WINPR_ATTR_UNUSED rdpTransport* t
 		case CONNECTION_STATE_FINALIZATION_CLIENT_GRANTED_CONTROL:
 		case CONNECTION_STATE_FINALIZATION_CLIENT_FONT_MAP:
 		default:
-			WLog_ERR(TAG, "%s state %d", rdp_get_state_string(rdp), rdp_get_state(rdp));
+			WLog_ERR(TAG, "%s state %" PRId32, rdp_get_state_string(rdp),
+			         WINPR_CXX_COMPAT_CAST(int32_t, rdp_get_state(rdp)));
 			break;
 	}
 
@@ -1247,12 +1258,13 @@ static BOOL freerdp_peer_send_server_redirection_pdu(freerdp_peer* peer,
 	WINPR_ASSERT(peer);
 	WINPR_ASSERT(peer->context);
 
-	wStream* s = rdp_send_stream_pdu_init(peer->context->rdp);
+	UINT16 sec_flags = 0;
+	wStream* s = rdp_send_stream_pdu_init(peer->context->rdp, &sec_flags);
 	if (!s)
 		return FALSE;
 	if (!rdp_write_enhanced_security_redirection_packet(s, redirection))
 		goto fail;
-	if (!rdp_send_pdu(peer->context->rdp, s, PDU_TYPE_SERVER_REDIRECTION, 0))
+	if (!rdp_send_pdu(peer->context->rdp, s, PDU_TYPE_SERVER_REDIRECTION, 0, sec_flags))
 		goto fail;
 	rc = rdp_reset_runtime_settings(peer->context->rdp);
 fail:
@@ -1439,24 +1451,14 @@ const char* freerdp_peer_os_minor_type_string(freerdp_peer* client)
 
 freerdp_peer* freerdp_peer_new(int sockfd)
 {
-	UINT32 option_value = 0;
-	socklen_t option_len = 0;
 	freerdp_peer* client = (freerdp_peer*)calloc(1, sizeof(freerdp_peer));
 
 	if (!client)
 		return NULL;
 
-	option_value = TRUE;
-	option_len = sizeof(option_value);
-
 	if (sockfd >= 0)
 	{
-		if (setsockopt(sockfd, IPPROTO_TCP, TCP_NODELAY, (void*)&option_value, option_len) < 0)
-		{
-			/* local unix sockets don't have the TCP_NODELAY implemented, so don't make this
-			 * error fatal */
-			WLog_DBG(TAG, "can't set TCP_NODELAY, continuing anyway");
-		}
+		(void)freerdp_tcp_set_nodelay(WLog_Get(TAG), WLOG_DEBUG, sockfd);
 	}
 
 	if (client)
